package com.jaywagnon.mymvc.persistence.repositories;


import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.springframework.beans.factory.annotation.Autowired;

import java.io.Serializable;
import java.util.List;


public abstract class AbstractRepository<T extends Serializable>
{
  private final Class<T> clazz;

  @Autowired
  SessionFactory sessionFactory;


  public AbstractRepository(Class<T> clazz)
  {
    this.clazz = clazz;
  }


  /**
   * Return an entity for the given ID.  Returns null if there is not entity
   * with the given ID.
   *
   * @param id
   * @return
   */
  public T findById(final Long id)
  {
    return (T) this.getCurrentSession().get(this.clazz, id);
  }


  /**
   * Return a List of entities for all the records.
   *
   * @return
   */
  public List<T> findAll()
  {
    return this.getCurrentSession().createQuery("from" + this.clazz.getName()).list();
  }


  /**
   * Create a record for the given entity.
   *
   * @param entity
   */
  public void create(final T entity)
  {
    if(entity != null)
      this.getCurrentSession().persist(entity);
  }


  /**
   * Update the record for the given entity.
   *
   * @param entity
   */
  public void update(final T entity)
  {
    if(entity != null)
      this.getCurrentSession().merge(entity);
  }


  /**
   * Delete the record for the given entity.
   *
   * @param entity
   */
  public void delete(final T entity)
  {
    if(entity != null)
      this.getCurrentSession().delete(entity);
  }


  /**
   * Delete the record for the entity for the given ID, if it exists.
   * Does nothing if there is no entity for the given ID.
   *
   * @param id
   */
  public void deleteById(final Long id)
  {
    delete(findById(id));
  }


  /**
   * Returns the current session from the factory.
   *
   * @return
   */
  protected final Session getCurrentSession()
  {
    return this.sessionFactory.getCurrentSession();
  }
}
