package com.jaywagnon.mymvc.services.impl;


import com.jaywagnon.mymvc.model.Person;
import com.jaywagnon.mymvc.persistence.entities.PersonEntity;
import com.jaywagnon.mymvc.persistence.repositories.PersonRepository;
import com.jaywagnon.mymvc.services.PersonService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;


@Service
public class PersonServiceImpl
        implements PersonService
{
  @Resource
  private PersonRepository repository;


  public Person find(final Long id)
  {
    if(id == null)
      return null;

    PersonEntity entity = repository.findById(id);
    return entity == null ? null : personFromEntity(entity);
  }


  public List<Person> findAll()
  {
    List<Person> list = new ArrayList<>();
    for(PersonEntity entity : repository.findAll()) {
      list.add(personFromEntity(entity));
    }
    return list;
  }


  public void create(final Person person)
  {
    if(person != null)
      repository.create(entityFromPerson(person));
  }


  public void update(final Person person)
  {
    if(person != null)
      repository.update(entityFromPerson(person));
  }


  public void delete(final Long id)
  {
    if(id != null)
      repository.deleteById(id);
  }


  public void delete(final Person person)
  {
    if(person != null)
      repository.delete(entityFromPerson(person));
  }


  private Person personFromEntity(PersonEntity entity)
  {
    Person person = new Person();
    person.setId(entity.getId());
    person.setName(entity.getName());
    return person;
  }


  private PersonEntity entityFromPerson(Person person)
  {
    PersonEntity entity = new PersonEntity();
    entity.setId(person.getId());
    entity.setName(person.getName());
    return entity;
  }
}
