package com.jaywagnon.mymvc.configuration;


import org.apache.commons.configuration.MapConfiguration;
import org.hibernate.SessionFactory;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.mockito.Spy;
import org.mockito.runners.MockitoJUnitRunner;
import org.springframework.dao.annotation.PersistenceExceptionTranslationPostProcessor;
import org.springframework.orm.hibernate4.HibernateTransactionManager;
import org.springframework.orm.hibernate4.LocalSessionFactoryBean;

import javax.sql.DataSource;
import java.util.Properties;

import static com.jaywagnon.mymvc.configuration.ConfigurationEnum.*;
import static junit.framework.Assert.assertNotNull;
import static junit.framework.TestCase.assertEquals;
import static junit.framework.TestCase.assertTrue;
import static org.mockito.Mockito.*;


@RunWith(MockitoJUnitRunner.class)
public class DataSourceConfigurationTest
{
  private static MapConfiguration configuration;

  @Spy
  DataSourceConfiguration dataSourceConfiguration;

  @Mock
  DataSource dataSource;

  @Mock
  Properties hibernateProperties;

  @Mock
  SessionFactory sessionFactory;


  @BeforeClass
  public static void setup()
  {
    configuration = mock(MapConfiguration.class);
    ConfigurationEnum.setConfiguration(configuration);
  }


  @Before
  public void init()
  {
    MockitoAnnotations.initMocks(this);
  }


  @Test
  public void test_dataSource()
  {
    DataSource dataSource = dataSourceConfiguration.dataSource();
    assertNotNull(dataSource);

    verify(configuration).getString(DATA_SOURCE_URL.toString());
    verify(configuration).getString(DATA_SOURCE_DRIVER.toString());
    verify(configuration).getString(DATA_SOURCE_USER.toString());
    verify(configuration).getString(DATA_SOURCE_PASSWORD.toString());
  }


  @Test
  public void test_getHibernateProperties()
  {
    Properties properties = dataSourceConfiguration.getHibernateProperties();
    assertNotNull(properties);
    assertEquals(3, properties.size());
    assertTrue(properties.containsKey("hibernate.dialect"));
    assertTrue(properties.containsKey("hibernate.hbm2ddl.auto"));
    assertTrue(properties.containsKey("hibernate.show_sql"));
  }


  @Test
  public void test_sessionFactory()
  {
    doReturn(dataSource).when(dataSourceConfiguration).dataSource();
    doReturn(hibernateProperties).when(dataSourceConfiguration).getHibernateProperties();
    LocalSessionFactoryBean factory = dataSourceConfiguration.sessionFactory();

    assertNotNull(factory);
    assertEquals(hibernateProperties, factory.getHibernateProperties());

    verify(dataSourceConfiguration).dataSource();
    verify(dataSourceConfiguration).getHibernateProperties();
  }


  @Test
  public void test_exceptionTranslation()
  {
    PersistenceExceptionTranslationPostProcessor processor = dataSourceConfiguration.exceptionTranslation();
    assertNotNull(processor);
  }


  @Test
  public void test_transactionManager()
  {
    HibernateTransactionManager manager = dataSourceConfiguration.transactionManager(sessionFactory);
    assertNotNull(manager);
    assertEquals(sessionFactory, manager.getSessionFactory());
  }
}
