package com.jaywagnon.mymvc.persistence.repositories;


import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import java.util.ArrayList;

import static junit.framework.TestCase.assertNotNull;
import static org.mockito.Matchers.eq;
import static org.mockito.Mockito.*;


@RunWith(MockitoJUnitRunner.class)
public class AbstractRepositoryTest
{
  @Mock
  SessionFactory sessionFactory;

  @Mock
  Session session;

  @InjectMocks
  MockRepository repository;


  @Mock
  MockEntity entity;


  @Before
  public final void setup()
  {
    when(sessionFactory.getCurrentSession()).thenReturn(session);
  }


  @Test
  public void test_Constructor()
  {
    assertNotNull(repository);
  }


  @Test
  public void test_find()
  {
    Long id = new Long(1);
    when(session.get(eq(MockEntity.class), anyLong())).thenReturn(entity);
    repository.findById(id);
    verify(session).get(eq(MockEntity.class), eq(id));
  }


  @Test
  public void test_findAll()
  {
    Query query = mock(Query.class);
    when(query.list()).thenReturn(new ArrayList<MockEntity>());

    when(session.createQuery(eq("from" + MockEntity.class.getName()))).thenReturn(query);
    repository.findAll();
    verify(session).createQuery(eq("from" + MockEntity.class.getName()));
  }


  @Test
  public void test_create()
  {
    repository.create(entity);
    verify(session).persist(entity);
  }


  @Test
  public void test_create_null()
  {
    repository.create(null);
    verify(session, never()).persist(entity);
  }


  @Test
  public void test_update()
  {
    repository.update(entity);
    verify(session).merge(entity);
  }


  @Test
  public void test_update_null()
  {
    repository.update(null);
    verify(session, never()).merge(entity);
  }


  @Test
  public void test_delete()
  {
    repository.delete(entity);
    verify(session).delete(entity);
  }


  @Test
  public void test_delete_null()
  {
    repository.delete(null);
    verify(session, never()).delete(entity);
  }


  @Test
  public void test_deleteById()
  {
    Long id = new Long(1);
    when(session.get(eq(MockEntity.class), anyLong())).thenReturn(entity);

    repository.deleteById(id);
    verify(session).delete(entity);
  }


  @Test
  public void test_deleteById_null()
  {
    Long id = new Long(1);
    when(session.get(eq(MockEntity.class), anyLong())).thenReturn(null);

    repository.deleteById(id);
    verify(session, never()).delete(entity);
  }
}
