package com.jaywagnon.mymvc.services.impl;


import com.jaywagnon.mymvc.model.Person;
import com.jaywagnon.mymvc.persistence.entities.PersonEntity;
import com.jaywagnon.mymvc.persistence.repositories.PersonRepository;
import com.jaywagnon.mymvc.services.PersonService;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.ArgumentCaptor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import java.util.ArrayList;
import java.util.List;

import static junit.framework.Assert.assertEquals;
import static junit.framework.TestCase.*;
import static org.mockito.Matchers.any;
import static org.mockito.Matchers.eq;
import static org.mockito.Mockito.*;


@RunWith(MockitoJUnitRunner.class)
public class PersonServiceImplTest
{
  private static PersonEntity entity1 = new PersonEntity(new Long(1), "One");
  private static PersonEntity entity2 = new PersonEntity(new Long(2), "Two");
  private static List<PersonEntity> entityList;

  private static Person person1 = new Person(entity1.getId(), entity1.getName());
  private static Person person2 = new Person(entity2.getId(), entity2.getName());
  private static List<Person> personList;


  @Mock
  private PersonRepository repository;

  @InjectMocks
  private PersonServiceImpl service;


  @BeforeClass
  public static void setupBeforeClass()
  {
    entityList = new ArrayList<>();
    entityList.add(entity1);
    entityList.add(entity2);

    personList = new ArrayList<>();
    personList.add(person1);
    personList.add(person2);
  }


  @Before
  public void setupBeforeTest()
  {
    when(repository.findById(entity1.getId())).thenReturn(entity1);
    when(repository.findById(entity2.getId())).thenReturn(entity2);
    when(repository.findAll()).thenReturn(entityList);
  }


  @Test
  public void test_Constructor()
  {
    assertNotNull(service);
    assertTrue(service instanceof PersonService);
  }


  @Test
  public void test_find()
  {
    service.find(person1.getId());
    verify(repository).findById(person1.getId());
  }


  @Test
  public void test_find_null()
  {
    assertNull(service.find(null));
    verify(repository, never()).findById(null);
  }


  @Test
  public void test_find_not_found()
  {
    Long id = new Long(100);
    assertNull(service.find(id));
    verify(repository).findById(id);
  }


  @Test
  public void test_findAll()
  {
    List<Person> persons = service.findAll();
    verify(repository).findAll();
    assertNotNull(persons);
    assertEquals(2, persons.size());
  }


  @Test
  public void test_create()
  {
    service.create(person1);
    ArgumentCaptor<PersonEntity> entityArgument = ArgumentCaptor.forClass(PersonEntity.class);
    verify(repository).create(entityArgument.capture());
    assertEquals(person1.getId(), entityArgument.getValue().getId());
    assertEquals(person1.getName(), entityArgument.getValue().getName());

  }


  @Test
  public void test_create_null()
  {
    service.create(null);
    verify(repository, never()).create(any(PersonEntity.class));
  }


  @Test
  public void test_update()
  {
    service.update(person1);
    ArgumentCaptor<PersonEntity> entityArgument = ArgumentCaptor.forClass(PersonEntity.class);
    verify(repository).update(entityArgument.capture());
    assertEquals(person1.getId(), entityArgument.getValue().getId());
    assertEquals(person1.getName(), entityArgument.getValue().getName());
  }


  @Test
  public void test_update_null()
  {
    service.update(null);
    verify(repository, never()).update(any(PersonEntity.class));
  }


  @Test
  public void test_delete_by_id()
  {
    service.delete(person1.getId());
    verify(repository).deleteById(person1.getId());
  }


  @Test
  public void test_delete_by_id_null()
  {
    service.delete((Long)null);
    verify(repository, never()).deleteById(person1.getId());
  }


  @Test
  public void test_delete()
  {
    service.delete(person1);
    ArgumentCaptor<PersonEntity> entityArgument = ArgumentCaptor.forClass(PersonEntity.class);
    verify(repository).delete(entityArgument.capture());
    assertEquals(person1.getId(), entityArgument.getValue().getId());
    assertEquals(person1.getName(), entityArgument.getValue().getName());

  }


  @Test
  public void test_delete_null()
  {
    service.delete((Person) null);
    verify(repository, never()).delete(any(PersonEntity.class));
  }
}
